--- bitops.c	Mon Jun 30 03:25:55 1997
+++ new/bitops.c	Wed Aug 06 23:18:45 1997
@@ -1,56 +1,35 @@
 /*
- * bitops.h --- Bitmap frobbing code.
+ * bitops.c --- Bitmap frobbing code.  See bitops.h for the inlined
+ * 	routines.
  * 
- * Copyright (C) 1993, 1994 Theodore Ts'o.  This file may be
- * redistributed under the terms of the GNU Public License.
+ * Copyright (C) 1993, 1994, 1995, 1996 Theodore Ts'o.
  * 
- * Taken from <asm/bitops.h>, Copyright 1992, Linus Torvalds.
+ * %Begin-Header%
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
+ * %End-Header%
  */
 
-#include "et/com_err.h"
+#include <stdio.h>
+#include <sys/types.h>
 
-#ifdef __GNUC__
-/*
- * Some hacks to defeat gcc over-optimizations..
- */
-struct __dummy_h { unsigned long a[100]; };
-#define ADDR (*(struct __dummy_h *) addr)
-#define CONST_ADDR (*(const struct __dummy_h *) addr)	
-
-int set_bit(int nr, void * addr)
-{
-	int oldbit;
-
-	__asm__ __volatile__("btsl %2,%1\n\tsbbl %0,%0"
-		:"=r" (oldbit),"=m" (ADDR)
-		:"r" (nr));
-	return oldbit;
-}
-
-int clear_bit(int nr, void * addr)
-{
-	int oldbit;
-
-	__asm__ __volatile__("btrl %2,%1\n\tsbbl %0,%0"
-		:"=r" (oldbit),"=m" (ADDR)
-		:"r" (nr));
-	return oldbit;
-}
+#include <linux/ext2_fs.h>
 
-int test_bit(int nr, const void * addr)
-{
-	int oldbit;
+#include "ext2fs.h"
 
-	__asm__ __volatile__("btl %2,%1\n\tsbbl %0,%0"
-		:"=r" (oldbit)
-		:"m" (CONST_ADDR),"r" (nr));
-	return oldbit;
-}
+#ifndef _EXT2_HAVE_ASM_BITOPS_
 
-#else
+/*
+ * For the benefit of those who are trying to port Linux to another
+ * architecture, here are some C-language equivalents.  You should
+ * recode these in the native assmebly language, if at all possible.
+ *
+ * C language equivalents written by Theodore Ts'o, 9/26/92.
+ * Modified by Pete A. Zaitcev 7/14/95 to be portable to big endian
+ * systems, as well as non-32 bit systems.
+ */
 
-#if 0
-int set_bit(int nr,void * addr)
+int ext2fs_set_bit(int nr,void * addr)
 {
 	int		mask, retval;
 	unsigned char	*ADDR = (unsigned char *) addr;
@@ -62,7 +41,7 @@
 	return retval;
 }
 
-int clear_bit(int nr, void * addr)
+int ext2fs_clear_bit(int nr, void * addr)
 {
 	int		mask, retval;
 	unsigned char	*ADDR = (unsigned char *) addr;
@@ -74,7 +53,7 @@
 	return retval;
 }
 
-int test_bit(int nr, const void * addr)
+int ext2fs_test_bit(int nr, const void * addr)
 {
 	int			mask;
 	const unsigned char	*ADDR = (const unsigned char *) addr;
@@ -84,8 +63,7 @@
 	return ((mask & *ADDR) != 0);
 }
 
-#endif
-#endif
+#endif	/* !_EXT2_HAVE_ASM_BITOPS_ */
 
 void ext2fs_warn_bitmap(errcode_t errcode, unsigned long arg,
 			const char *description)
@@ -95,3 +73,14 @@
 	else
 		com_err(0, errcode, "#%u", arg);
 }
+
+void ext2fs_warn_bitmap2(ext2fs_generic_bitmap bitmap,
+			    int code, unsigned long arg)
+{
+	if (bitmap->description)
+		com_err(0, bitmap->base_error_code+code,
+			"#%u for %s", arg, bitmap->description);
+	else
+		com_err(0, bitmap->base_error_code + code, "#%u", arg);
+}
+
