#include <windows.h>
#include <winioctl.h>
#include <stdio.h>

#define ONE_MB 1048576 // 1024 * 1024

//
// PrintErrMessage
//
void PrintErrMessage(LPSTR funcName, DWORD LastMess) {

	LPVOID lpMsgBuf;

	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 
				NULL,
				LastMess, 
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
				(LPTSTR) &lpMsgBuf,
				0, NULL);

	printf("%s : %s\n",funcName, lpMsgBuf);
	LocalFree( lpMsgBuf ); 
}

//
// GetDiskGeometry
//
BOOL GetDiskGeometry(HANDLE hDisk, PDISK_GEOMETRY lpGeometry) {
    
	DWORD ReturnedByteCount;

    return DeviceIoControl(hDisk, IOCTL_DISK_GET_DRIVE_GEOMETRY, NULL, 0,
                lpGeometry, sizeof(*lpGeometry), &ReturnedByteCount,NULL);
}


//
// PrintGeometry
//
VOID PrintGeometry(PDISK_GEOMETRY lpGeometry) {
    
	printf("Size = %d Mb - ", lpGeometry->Cylinders.LowPart * lpGeometry->SectorsPerTrack * 
		lpGeometry->TracksPerCylinder * 512 / ONE_MB);
    printf("Cyl = %d - Tr/Cyl = %d - Sec/Tr = %d\n", lpGeometry->Cylinders.LowPart,
        lpGeometry->TracksPerCylinder, lpGeometry->SectorsPerTrack);
}


//
// GetPartitionTable
//
BOOL GetPartitionTable(HANDLE hDisk, DRIVE_LAYOUT_INFORMATION * lpLayout, DWORD taille) {
    
	DWORD ReturnedByteCount;

    return  DeviceIoControl(hDisk, IOCTL_DISK_GET_DRIVE_LAYOUT, NULL, 0, 
			lpLayout, taille, &ReturnedByteCount, NULL);

//	printf("PartitionCount= %lu\n", lpLayout->PartitionCount);
//	printf("Disk signature = %d\n", lpLayout->Signature);
}

//
// PrintPartitionTable
//
VOID PrintPartitionTable(PDRIVE_LAYOUT_INFORMATION lpLayout) {
    
	PPARTITION_INFORMATION part = NULL;
	DWORD i;

	printf("Number\tSize\tType\n");

	for (i = 0; i < lpLayout->PartitionCount; i++) {
		part = &lpLayout->PartitionEntry[i];
		if (!part->PartitionType) continue; // not a valid partition type, go on
		printf("%u\t%u\t0x%.2x\n",part->PartitionNumber,
			part->PartitionLength.LowPart / ONE_MB, part->PartitionType);
	}
}


//
// Main
//
DWORD _cdecl main(int argc, char *argv[], char *envp[]) {
    
	char Drive[MAX_PATH];
    HANDLE hDrive;
    DISK_GEOMETRY Geometry;
	DRIVE_LAYOUT_INFORMATION * lpLayout;
	DWORD taille;
      
	if (argc < 2) {
		printf("Essai PhysicalDriveNumber (starts at 0)\n");
		ExitProcess(0);
		return(0);
	}

    sprintf(Drive,"\\\\.\\PHYSICALDRIVE%s",argv[1]);
	printf("\nPhysical Drive %s specifications\n-------------------------------\n", argv[1]);
	hDrive = CreateFile(Drive, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
    if ( hDrive == INVALID_HANDLE_VALUE ) {
       PrintErrMessage("Open", GetLastError());
       ExitProcess(1);
    }

	// Disk Geometry part
	if ( !GetDiskGeometry(hDrive,&Geometry) ) {
       PrintErrMessage("GetDiskGeometry", GetLastError());
       ExitProcess(1);
    }
	PrintGeometry(&Geometry);

	taille = sizeof(DRIVE_LAYOUT_INFORMATION) + 10 * sizeof(PARTITION_INFORMATION);
	lpLayout = (DRIVE_LAYOUT_INFORMATION *) malloc(taille);
  

	// Disk partition part
	printf("\nPartition Table\n---------------\n");
	if ( !GetPartitionTable(hDrive,lpLayout, taille) ) {
       PrintErrMessage("GetPartitionTable", GetLastError());
       ExitProcess(1);
    }
    PrintPartitionTable(lpLayout);

    ExitProcess(0);
    return (0);
}
